/* $Workfile: A16Vd5x4.c $       $Revision: 23 $      $Modtime: 10/02/02 6:38p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    A16Vd5x4.c

    A simple example console application demonstrating the use of OTX Driver
    with the Arni-16-PCI board and Vidar-5x4-ASM Daughter Board.

    The sample application configures 16 POTS Lines and one Handset and provides
    a simple console interface to manage the POTS Line Interface (LI) devices.
    The user can use the following simple keystrokes as commands to go off-hook
    and on-hook, to dial numbers, and to connect a POTS line to the handset.

    The main command keys are listed in the following:
     - Press 'h' or '?' for help. 'q' to quit.
     - Press '+' or '-' to select a active POTS Line (0-15)
     - Press 'f' to make the active POTS line go off-hook
     - Press 'n' to make the active POTS line go on-hook
     - Press 'l' for hook flash on the active POTS line
     - Press 't' to turn sweeping sine wave on and off on the active POTS line


*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
#include "OtxErr.h"
#include "OtxType.h"
#include "OtxDrv.h"
#include "OtxBrd.h"
#include "OtxPots.h"
#include "OtxTone.h"
#include "OtxDsp.h"
#include "OtxData.h"

/*******************************************************************************
  Macros, Types
*******************************************************************************/
// Include one of the following if depending if you want to run the demo w
// with 2 or 16 Line Interfaces LIs
#if 1
  #define DEMO_POTS_LI_MAX       16        // Number of POTS lines supported by the Demo
#else
  #define DEMO_POTS_LI_MAX        2        // Number of POTS lines supported by the Demo
#endif

#define DEMO_POTS_LI_PER_DSP    4          // Number of POTS lines supported by DSP
#define DEMO_CODEC_MAX          1          // Arni Board contains 4 codecs. Use 1 in this demo

// Number of LI Controllers needed. Each supports 8 Line Interfaces
#if (DEMO_POTS_LI_MAX > 8)
  #define DEMO_POTS_CTRL_MAX  2
#else
  #define DEMO_POTS_CTRL_MAX  1
#endif

// Number of DSPs needed
#if (DEMO_POTS_LI_MAX % DEMO_POTS_LI_PER_DSP)
  #define DEMO_DSP_MAX ((DEMO_POTS_LI_MAX / DEMO_POTS_LI_PER_DSP) + 1)
#else
  #define DEMO_DSP_MAX (DEMO_POTS_LI_MAX / DEMO_POTS_LI_PER_DSP)
#endif

#define DEMO_TASK_REF           1L            // Task Reference for a non-blocking call
#define DEMO_DEFAULT_LOG_FILE   "A16Vd5x4.log"  // Name of the LOG file
#define DEMO_DEFAULT_BOARD_NO   0

typedef struct DemoEnvT {
  char           *logFileName;
  int             boardNo;
  OtxDataFormatE  law;
  OtxPotsCfgSetE  nSetId;
} DemoEnvT;


/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoCreateEventQueue(void);
static void DemoOpenPhysicalDevices(void);
static void DemoEnablePhysicalDevices(void);
static void DemoCreateLogicalDevices(void);
static void DemoConnectDevices(void);
static void DemoInitializeLogicalDevices(void);
static void DemoEnableLogicalDevices(void);
static void DemoPrintEvent(IN OtxEventDataS *pEventData);
static void DemoCheckRc(IN OTX_RESULT otxRc);
static void DemoLogf(char *formatStr, ...);
static void DemoHelpPanel(void);
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);


/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue        = OTX_INVALID_HANDLE_VALUE;
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;

static OsaMutexS g_demoLogfCritSec;
static OsaMutexS g_demoCallStateTransitionCritSec;

struct {  // Handles to Physical Devices
  OTX_HANDLE hBoardController0;
  OTX_HANDLE hAsmController0;
  OTX_HANDLE hLiController[2];
  OTX_HANDLE hLi[16];
  OTX_HANDLE hDsp[4];
  OTX_HANDLE hCodec[4];
  OTX_HANDLE hTss;
} g_hPhysDevs;



struct {  // Handles to Logical Devices
  OTX_HANDLE hDtmfDetector[DEMO_POTS_LI_MAX];
  OTX_HANDLE hToneGenerator[DEMO_POTS_LI_MAX];
  OTX_HANDLE hDtmfDialler[DEMO_POTS_LI_MAX];
  OTX_HANDLE hRxDataConverter[DEMO_POTS_LI_MAX];
  OTX_HANDLE hTxDataConverter[DEMO_POTS_LI_MAX];

  struct {
    OTX_HANDLE  hHandle;
    OTX_EVENT   hNotificationEvent;
    OTX_HANDLE  hEventQueue;
    OTX_INT32   nSourceId;

  } mDualToneDetector[DEMO_POTS_LI_MAX];

} g_hLogicDevs;


// Demo Call State (DCS)
typedef enum {
  DCS_SILENT     = 0,  // Silent, default state
  DCS_RING_PEND1 = 1,  // First Ring tone detected, waiting for pause
  DCS_RING_PEND2 = 2,  // First Ring tone and Pause detected, waiting for second ring tone
  DCS_RING       = 3,  // Ringing (Ring + Pause + Ring detected)
  DCS_BUSY_PEND1 = 4,  // First Busy tone detected, waiting for second busy tone
  DCS_BUSY_PEND2 = 5,  // First Busy tone and Pause detected, waiting for second busy tone
  DCS_BUSY       = 6,  // Busy (Busy + Pause + Busy detected)
  DCS_DIAL       = 7   // Dial tone detected
} DemoCallStateT;

// Demo Call State Event
typedef enum {
  DCSE_TIMER     = 0,
  DCSE_PAUSE     = 1,
  DCSE_RING_TONE = 2,
  DCSE_BUSY_TONE = 3,
  DCSE_DIAL_TONE = 4
} DemoCallStateEventT;


DemoCallStateT g_eCurrentCallState[DEMO_POTS_LI_MAX];

struct {  // Handles to Pipes
  OTX_HANDLE hLiDsp[DEMO_POTS_LI_MAX];
  OTX_HANDLE hDspLi[DEMO_POTS_LI_MAX];
  OTX_HANDLE hDspCodec[DEMO_POTS_LI_MAX];
  OTX_HANDLE hCodecDsp[DEMO_POTS_LI_MAX];
} g_hPipes;

char *g_sqNumber0 = "102";

static OTX_BOOL  g_bSineWaveOn[DEMO_POTS_LI_MAX];
//static OTX_INT32 g_nDsp;            // Number of DSPs needed

/*******************************************************************************
*******************************************************************************/


void DemoCreateEventQueue(void)
/*******************************************************************************
  @func
    Create the event queue to be used by the driver to inform the application
    about events, such as line status changes, or detected DMTF digits.
*******************************************************************************/
{
  OTX_RESULT result;

  // Create a notification Event
  result = OtxDrvCreateEvent(&g_hNotificationEvent); DemoCheckRc(result);

  // Create Event Queue for driver notifications
  result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(result);

}



void DemoOpenPhysicalDevices(void)
/*******************************************************************************
  @func
    Opens the Physical Devices on the Arni-16-PCI board

  @comm
    Calls function <f OtxDrvOpenPhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.
*******************************************************************************/
{
  OTX_RESULT result;
  OTX_INT32 j;
  OTX_INT32 i;
  OTX_INT32 nLi = 0;
  OTX_INT32 nLiMax = 8;   // Max 8 POTS Li's per LI Controller
  OTX_INT32 nLiOpened = 0;
  OTX_INT32 nDsp;

  // Get Handle to the Physical ARNI Board Controller
  DemoLogf("Open Board OTX_DEVICE_ARNI_PCI\n");
  result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_ARNI_PCI, g_demoEnv.boardNo, g_demoEnv.boardNo, g_hEventQueue, &g_hPhysDevs.hBoardController0); DemoCheckRc(result);

  // Get a handle to the Time-Space Switch (TSS) on the Thor-2-PCI board
  DemoLogf("Open OTX_DEVICE_TSS\n");
  result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_TSS, 0, 0, g_hEventQueue, &g_hPhysDevs.hTss); DemoCheckRc(result);

  // Connect to the POTS LI (0-7) Controller
  for (j = 0; j < DEMO_POTS_CTRL_MAX; j++) {
    DemoLogf("Open OTX_DEVICE_LI_CONTROLLER  #%d (AIC)\n", j);
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_LI_CONTROLLER, j, j, g_hEventQueue, &g_hPhysDevs.hLiController[j]); DemoCheckRc(result);

    DemoLogf("Configuring AIC#%d for country set=");
    switch (g_demoEnv.nSetId) {
    case OTX_POTS_CFG_SET_USA:
      DemoLogf("USA");
      break;
    case OTX_POTS_CFG_SET_GERMANY: 
      DemoLogf("Germany");
      break;
    case OTX_POTS_CFG_SET_UK:
      DemoLogf("United Kingdom");
      break;
    case OTX_POTS_CFG_SET_FRANCE:
      DemoLogf("France");
      break;
    case OTX_POTS_CFG_SET_AUSTRALIA:
      DemoLogf("Australia");
      break;
    case OTX_POTS_CFG_SET_JAPAN:
      DemoLogf("Japan");
      break;
    default:
      DemoLogf("UNKNOWN");
      break;
    }
    DemoLogf("\n");
  
    result = OtxDrvSetAttributeValueUint32(g_hPhysDevs.hLiController[j], OTX_ATTR_POTS_CFG_SET, (OTX_UINT32)g_demoEnv.nSetId); DemoCheckRc(result);

    // Get handles to POTS line devices
    if (DEMO_POTS_LI_MAX - nLiOpened > 8) {
      nLiMax = 8;
    }  else {
      nLiMax = DEMO_POTS_LI_MAX - nLiOpened;
    }
    for (i = 0 ; i < nLiMax; i++) {
      DemoLogf("Open OTX_DEVICE_LI_POTS  #%d \n", i);
      result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hLiController[j], OTX_DEVICE_LI_POTS, i, j*nLiMax + i, g_hEventQueue, &g_hPhysDevs.hLi[nLi]); DemoCheckRc(result);
      nLi++;
      nLiOpened++;
    }

  }

  // Get a handle to the Codec Devices
  for (i = 0; i < DEMO_CODEC_MAX; i++) {
    DemoLogf("Open OTX_DEVICE_CODEC\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_CODEC, i, i, g_hEventQueue, &g_hPhysDevs.hCodec[i]); DemoCheckRc(result);
  }

  // Get a handle to to the ASM Daughter Board and one DSP on it.
  DemoLogf("Open OTX_DEVICE_VIDAR_5x_ASM\n");
  result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hBoardController0, OTX_DEVICE_VIDAR_5x_ASM, 0, 0, g_hEventQueue, &g_hPhysDevs.hAsmController0); DemoCheckRc(result);

  for (i = 0; i < DEMO_DSP_MAX; i++) {
    DemoLogf("Open OTX_DEVICE_DSP\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hAsmController0, OTX_DEVICE_DSP, i, i, g_hEventQueue, &g_hPhysDevs.hDsp[i]); DemoCheckRc(result);
  }

  for (nDsp = 0; nDsp < DEMO_DSP_MAX; nDsp++) {
    DemoLogf("Loading OTX_DSP_SPM_OTXSPM2 to DSP#%d \n", nDsp);
    result = OtxDspRunSpm(g_hPhysDevs.hDsp[nDsp], OTX_DSP_SPM_OTXSPM2); DemoCheckRc(result);
  }

}


/*******************************************************************************
  @func
    Initialize the physical devices that need it
*******************************************************************************/
void DemoInitializePhysicalDevices()
{
  // Nothing in this demo program
}


void DemoEnablePhysicalDevices(void)
/*******************************************************************************
  @func
    Enable Configured Physical Devices
*******************************************************************************/
{
  OTX_RESULT result;
  OTX_INT32  i;

  DemoLogf("In DemoEnablePhysicalDevices\n");
  result = OtxDrvEnable(g_hPhysDevs.hBoardController0,     OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  result = OtxDrvEnable(g_hPhysDevs.hAsmController0,       OTX_TASK_SYNC, 5000L); DemoCheckRc(result);

  for (i = 0; i < DEMO_DSP_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hDsp[i],            OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

  for (i = 0; i < DEMO_POTS_CTRL_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLiController[i],   OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }
    
  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLi[i],             OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

  for (i = 0; i < DEMO_CODEC_MAX; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hCodec[i],          OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

}


/*******************************************************************************
  @func
    Create the needed Logical Devices

  @comm
    The logical devices are different from the phyiscal devices in that the
    the number of logical devices is not fixed. More of them can be requested
    by the user and allocated by the system if needed.
*******************************************************************************/
void DemoCreateLogicalDevices(
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_INT32  nDsp;
  OTX_INT32  nLi;
  OTX_INT32  nLiSupportDevsOpened = 0;
  OTX_INT32  nLiSupportDevsToOpen;

  // Load DSP Program
  for (nDsp = 0; nDsp < DEMO_DSP_MAX; nDsp++) {

    // Get handles to POTS line devices
    if (DEMO_POTS_LI_MAX - nLiSupportDevsOpened > DEMO_POTS_LI_PER_DSP) {
      nLiSupportDevsToOpen = DEMO_POTS_LI_PER_DSP;
    }  else {
      nLiSupportDevsToOpen = DEMO_POTS_LI_MAX - nLiSupportDevsOpened;
    }

    for (i = 0; i < nLiSupportDevsToOpen; i++) {

      nLi = (nDsp * DEMO_POTS_LI_PER_DSP) + i;

      // Create Dtmf Detector Logical Devices
      DemoLogf("Create Logical Device OTX_LDEVICE_TONE_DTMF_DETECTOR #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice( g_hPhysDevs.hDsp[nDsp], OTX_LDEVICE_TONE_DTMF_DETECTOR, g_hEventQueue, nLi, &g_hLogicDevs.hDtmfDetector[nLi]); DemoCheckRc(result);

      // Set the minimum required duration for a DTMF tone to be detected
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.hDtmfDetector[nLi], OTX_ATTR_DTMF_DETECTOR_MIN_DURATION, 20 /* milliseconds */); DemoCheckRc(result);



      // Create Dial, Busy, and Ring Tone Detector Logical Devices (which is implemented as generic Dual Tone Detector)
      // Create a notification event for the Dual Tone Detector
      // Each Dual Tone Detector has its own EventQueue
      result = OtxDrvCreateEvent(
        &g_hLogicDevs.mDualToneDetector[nLi].hNotificationEvent
        ); 
      DemoCheckRc(result);

      // Create Event Queue for driver notifications
      result = OtxDrvCreateEventQueue(
        128, 
        g_hLogicDevs.mDualToneDetector[nLi].hNotificationEvent, 
        &g_hLogicDevs.mDualToneDetector[nLi].hEventQueue
        ); 
      DemoCheckRc(result);


      DemoLogf("Create Logical Device OTX_LDEVICE_TONE_DUAL_DETECTOR #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice( 
        g_hPhysDevs.hDsp[nDsp], 
        OTX_LDEVICE_TONE_DUAL_DETECTOR, 
        g_hLogicDevs.mDualToneDetector[nLi].hEventQueue, 
        nLi, 
        &g_hLogicDevs.mDualToneDetector[nLi].hHandle
        ); 
      DemoCheckRc(result);

      // Create Tone Effect Generators Logical Devices
      DemoLogf("Create Logical Device OTX_LDEVICE_TONE_EFFECTS_GENERATOR #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice( g_hPhysDevs.hDsp[nDsp], OTX_LDEVICE_TONE_EFFECTS_GENERATOR, g_hEventQueue, nLi, &g_hLogicDevs.hToneGenerator[nLi]); DemoCheckRc(result);

      // Create DTMF Dialers Logical Devices
      DemoLogf("Create Logical Device OTX_LDEVICE_TONE_DTMF_DIALER #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp[nDsp], OTX_LDEVICE_TONE_DTMF_DIALER, g_hEventQueue, nLi, &g_hLogicDevs.hDtmfDialler[nLi]); DemoCheckRc(result);

      // Create Data Converters for converting from 16-bit LI Data to 8-bit u-law/A-law for Codec
      DemoLogf("Create Logical Device OTX_LDEVICE_DATA_CONVERTER #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp[nDsp], OTX_LDEVICE_DATA_CONVERTER, g_hEventQueue, nLi, &g_hLogicDevs.hRxDataConverter[nLi]); DemoCheckRc(result);
      DemoLogf("Create Logical Device OTX_LDEVICE_DATA_CONVERTER #%d\n", nLi);
      result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp[nDsp], OTX_LDEVICE_DATA_CONVERTER, g_hEventQueue, nLi, &g_hLogicDevs.hTxDataConverter[nLi]); DemoCheckRc(result);

      nLiSupportDevsOpened++;

    }
  }

}


void DemoConnectDevices(
/*******************************************************************************
  @func
    Create the needed Pipe Devices and connect the physical and logical devices
    to pipes.

  @comm
    A Pipe Device is a logical entity representing a serial bit-stream (a subset of a
    Highway) with a data rate between 8 kbit/s and 2.048Mbit/s. A pipe is a characterized
    bu its capacity (in 8 kbit/s increments). The user can create a pipe with
    the driver function <f OtxDrvCreatePipe>. A pipe is connected to physical or
    logical devices and mapped to specific time-slots with the driver function
    <f OtxDrvConnectPipe>.
*******************************************************************************/
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_UINT8 sqTimeSlotMask[32];
  OTX_INT32 ts;

  DemoLogf("In DemoConnectDevices\n");
  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {
    // LiX <--> Dsp
    // Data from and to POTS LI's is 16 bits, requiring 128 kbit/s capacity
    result = OtxDrvCreatePipe(128, &g_hPipes.hLiDsp[i]); DemoCheckRc(result);
    result = OtxDrvCreatePipe(128, &g_hPipes.hDspLi[i]); DemoCheckRc(result);

    // Use TS 0-1, 2-3, 4-5, 6-7 on the DSP highway for the connection between the DSP and AIC
    ts = (i%DEMO_POTS_LI_PER_DSP) * 2;
    memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
    sqTimeSlotMask[ts + 0] = 0xFF;
    sqTimeSlotMask[ts + 1] = 0xFF;


    // Connect Pipes
    result = OtxDrvConnectPipe(g_hPipes.hLiDsp[i], OTX_PIPE_INPUT,  g_hPhysDevs.hLi[i], -1, NULL, 0); DemoCheckRc(result);
    result = OtxDrvConnectPipe(g_hPipes.hLiDsp[i], OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp[i/DEMO_POTS_LI_PER_DSP], -1, sqTimeSlotMask, 32); DemoCheckRc(result);
    result = OtxDrvConnectPipe(g_hPipes.hDspLi[i], OTX_PIPE_INPUT,  g_hPhysDevs.hDsp[i/DEMO_POTS_LI_PER_DSP], -1, sqTimeSlotMask, 32); DemoCheckRc(result);
    result = OtxDrvConnectPipe(g_hPipes.hDspLi[i], OTX_PIPE_OUTPUT, g_hPhysDevs.hLi[i], -1, NULL, 0); DemoCheckRc(result);
  }


  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {
    // DSP <--> Codec
    result = OtxDrvCreatePipe(64, &g_hPipes.hDspCodec[i]); DemoCheckRc(result);
    result = OtxDrvCreatePipe(64, &g_hPipes.hCodecDsp[i]); DemoCheckRc(result);

    // Use TS 16, 17, 18, 19 on the DSP highway for the connection between the DSP and Codec
    ts = (i%DEMO_POTS_LI_PER_DSP) + 16;
    memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
    sqTimeSlotMask[ts] = 0xFF;

    // Connect one end of all the Pipes to DSP
    result = OtxDrvConnectPipe(g_hPipes.hDspCodec[i], OTX_PIPE_INPUT,  g_hPhysDevs.hDsp[i/DEMO_POTS_LI_PER_DSP],     -1, sqTimeSlotMask, 32); DemoCheckRc(result);
    result = OtxDrvConnectPipe(g_hPipes.hCodecDsp[i], OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp[i/DEMO_POTS_LI_PER_DSP],     -1, sqTimeSlotMask, 32); DemoCheckRc(result);
  }

  // Connect POTS #0 (initially active) to Codec 0
  result = OtxDrvConnectPipe(g_hPipes.hDspCodec[0], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);
  result = OtxDrvConnectPipe(g_hPipes.hCodecDsp[0], OTX_PIPE_INPUT,  g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);


  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {

    // CONNECT LOGICAL DEVICES WITHIN THE DSP
    // --------------------------------------

    // Create the following Connections in receive direction:
    //  POTS Li  ----+---> DataConverter -----> Codec
    //               |
    //               +---> DtmfDetector
    //               |
    //               +---> DialtoneDetector
    result = OtxDrvConnectLogicalDevice(g_hPipes.hLiDsp[i], -1, g_hLogicDevs.hRxDataConverter[i],    -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hPipes.hLiDsp[i], -1, g_hLogicDevs.hDtmfDetector[i],       -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hPipes.hLiDsp[i], -1, g_hLogicDevs.mDualToneDetector[i].hHandle,   -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.hRxDataConverter[i], -1, g_hPipes.hDspCodec[i], -1); DemoCheckRc(result);

    // Make the following connections in transmit direction:
    //    Codec  ---> DataConverter ---+
    //                                 |
    //                ToneGenerator ---+--> POTS Li
    //                                 |
    //                DtmfDialler   ---+
    result = OtxDrvConnectLogicalDevice(g_hPipes.hCodecDsp[i], -1, g_hLogicDevs.hTxDataConverter[i], -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.hTxDataConverter[i], -1, g_hPipes.hDspLi[i], -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.hToneGenerator[i],   -1, g_hPipes.hDspLi[i], -1); DemoCheckRc(result);
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.hDtmfDialler[i],     -1, g_hPipes.hDspLi[i], -1); DemoCheckRc(result);
  }

}


void DemoInitializeLogicalDevices(
/*******************************************************************************
  @func
    Initialize the logical device. This function must be called after
    the devices are created and connected.
*******************************************************************************/
  void
)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_UINT32 nThreshold;

  DemoLogf("In DemoInitializeLogicalDevices\n");
  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {

    // Initialize the DTMF Detector
    result = OtxToneDtmfDetectorInit(
      g_hLogicDevs.hDtmfDetector[i], 
      OTX_DATA_FORMAT_UINT16, 
      OTX_TRUE); 
    DemoCheckRc(result);
    
    // Initialize the DialTone Detector
    {
      
      OTX_UINT32    sqFreq[4];       // An array of up to 6 frequencies to be detected.
      OTX_INT32     coFreq = 4;       // Number of elements in <p sqFreq>

      if (i == 0) {
        result = OtxDrvSetAttributeValueUint32(
          g_hLogicDevs.mDualToneDetector[i].hHandle, 
          OTX_ATTR_DUAL_TONE_DETECTOR_VERBOSE_STATUS, 
          0);  // 0 = No Verbose status messages
        DemoCheckRc(result);
      }


      if (i == 0) {
        
        result = OtxDrvGetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, &nThreshold); DemoCheckRc(result);
        DemoLogf("Dual Tone Detector[%d]: OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY  = %d\n", i, nThreshold);
        
        result = OtxDrvGetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, &nThreshold); DemoCheckRc(result);
        DemoLogf("Dual Tone Detector[%d]: OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY = %d\n", i, nThreshold);
        
        result = OtxDrvGetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST, &nThreshold); DemoCheckRc(result);
        DemoLogf("Dual Tone Detector[%d]: OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST        = %d\n", i, nThreshold);
        
        result = OtxDrvGetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD, &nThreshold); DemoCheckRc(result);
        DemoLogf("Dual Tone Detector[%d]: OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD = %d\n", i, nThreshold);
      }


      // Modifing default thresholds
      nThreshold = 600; // Min tone energy for first and second tones
      DemoLogf("Dual Tone Detector[%d]: Setting OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY  = %d\n", i, nThreshold);
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TONE_ENERGY, nThreshold); DemoCheckRc(result);
      
      nThreshold = 50;  // Max energy for pause
      DemoLogf("Dual Tone Detector[%d]: Setting OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY = %d\n", i, nThreshold);
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_PAUSE_ENERGY, nThreshold); DemoCheckRc(result);
      
      nThreshold = 300; // Max ratio HighestEnergy/2ndHighestEnergy*100 (225 = 2.25 times)
      // Note: We need a to accept a high ratio here since the frequencies for the
      // ring tone are very close to each other, which produces a large ratio between
      // the highest and the second highest energy level.
      DemoLogf("Dual Tone Detector[%d]: Setting OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST        = %d\n", i, nThreshold);
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_TWIST, nThreshold); DemoCheckRc(result);
      
      nThreshold = 30;  // Min ratio 2ndHighestEnergy/3rdHighestEnergy*100  (30 = 0.3 times)
      // Note: We need a to accept a low ratio here since the frequencies for the
      // dial tone are fairly low frequencies, which produces a relatively low ratio between
      // the second highest and the third highest energy level.
      DemoLogf("Dual Tone Detector[%d]: Setting OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD = %d\n", i, nThreshold);
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.mDualToneDetector[i].hHandle, OTX_ATTR_DUAL_TONE_DETECTOR_THRESHOLD_RELATIVE_3RD, nThreshold); DemoCheckRc(result);

      
      /*
      
          .         FreqA	 FreqB  FreqC  FreqD  FreqE	 FreqF 
          FreqA    { -1,    1,      2,     4,     7,    11  },
          FreqB    {  1,   -1,      3,     5,     8,    12  },
          FreqC    {  2,    3,     -1,     6,     9,    13  },
          FreqD    {  4,    5,      6,    -1,    10,    14  },
          FreqE    {  7,    8,      9,    10,    -1,    15  },
          FreqF    { 11,   12,     13,    14,    15,    -1  }

      */

      // We are looking at three different dual tones: dial, ring and busy tones.
      // We use the tone frequencies from the AT&T 5D5-900-106 specification
      // 350/440 is dial tone (dualtone index 1)  (constant on, no cadence)
      // 440/480 is ring tone (dualtone index 3)  (on 2.0sec, off 4.0sec)
      // 480/620 is busy tone (dualtone index 6)  (on 0.5sec, off 0.5sec)


      sqFreq[0] = 350; // Freq A in Hz
      sqFreq[1] = 440; // Freq B in Hz       
      sqFreq[2] = 500; // Freq C in Hz. We use a slightly higher detection frequency for the 480Hz tone since to better match the frequency bins of the Goertzel detection algorithm for the 12.75ms buffer.
      sqFreq[3] = 620; // Freq D in Hz       
      

      
      // Initialize the Generic Dual Tone Detector. Use a couple of DTMF tone frequencies as an example
      result =  OtxToneDualDetectorInit(
        g_hLogicDevs.mDualToneDetector[i].hHandle,           // Handle to the Logical device
        OTX_DATA_FORMAT_UINT16,    // Data Format of the Monitored Data Stream
        sqFreq,           // An array of up to 6 frequencies to be detected.
        coFreq,           // Number of elements in <p sqFreq>
        OTX_TRUE,         // If OTX_TRUE, generate events and store the received digits
        OTX_TRUE,         // If OTX_TRUE the detector generates an event when a tone ends
        200               // Minimum duration required to validate a dual tone
        );
      DemoCheckRc(result);
      
    }
    
    // Initialize the Tone Effect Generators
    result = OtxToneEffectsGeneratorInit(g_hLogicDevs.hToneGenerator[i], OTX_DATA_FORMAT_UINT16, OTX_TONE_SWEEPING_SINE, OTX_TONE_GAIN_0DB); DemoCheckRc(result);
    g_bSineWaveOn[i] = OTX_FALSE;
    
    // Initialize the DTMF Diallers
    result = OtxToneDtmfDiallerInit(g_hLogicDevs.hDtmfDialler[i], OTX_DATA_FORMAT_UINT16, 8, 8, OTX_TONE_GAIN_0DB); DemoCheckRc(result);
    
    // Initialize the Data Converters
    result = OtxDataConverterInit(g_hLogicDevs.hRxDataConverter[i], OTX_DATA_FORMAT_UINT16, g_demoEnv.law); DemoCheckRc(result);
    result = OtxDataConverterInit(g_hLogicDevs.hTxDataConverter[i], g_demoEnv.law, OTX_DATA_FORMAT_UINT16); DemoCheckRc(result);
  }

}




void DemoEnableLogicalDevices(void)
/*******************************************************************************
  @func
    Enable Configured Logical Devices
*******************************************************************************/
{
  OTX_RESULT result;
  OTX_INT32  i;

  DemoLogf("In DemoEnableLogicalDevices\n");

  for (i = 0; i < DEMO_POTS_LI_MAX; i++) {
    result = OtxDrvEnable(g_hLogicDevs.hDtmfDetector[i],  OTX_TASK_SYNC, 5000L); DemoCheckRc(result);

    result = OtxDrvEnable(g_hLogicDevs.mDualToneDetector[i].hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(result);

    result = OtxDrvEnable(g_hLogicDevs.hDtmfDialler[i],  OTX_TASK_SYNC, 5000L); DemoCheckRc(result);

    result = OtxDrvEnable(g_hLogicDevs.hRxDataConverter[i],  OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
    result = OtxDrvEnable(g_hLogicDevs.hTxDataConverter[i],  OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
  }

}


static void DemoCheckRc(IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  short c;
  if ( OTX_FAILURE(result) ) {
    DemoLogf("Error: rc = %X\n", result);

    DemoLogf("Do you want to exit (Y/N) ");
    c = getch();
    if (c == 'y' || c == 'Y') {
      // Delete allocated resources. IMPORTANT TO CALL BEFORE APPLICATION EXIT!! (for now, but not in future releases of the driver)
      OtxDrvReleaseLib();
			OsaConsoleExit();
      exit(1);
    }

  }
}


/*******************************************************************************
  @func
    Prints a timestamp in a MM-DD hh:mm:ss:mmm format, where

    MM  = month
    DD  = day (of month)
    hh  = hour of day
    mm  = minutes of day
    ss  = seconds of day
    mmm = milliseconds

*******************************************************************************/
void DemoPrintTimeStamp(
  OTX_DATETIME nTimeStamp
)
{
  OTX_DATETIME nTimeStampMs;
  OTX_DATETIME nTimeStampSec;
  struct tm *ptCurrentDayAndTime;
  time_t nCurrentTimeSecs;
  
  // Subtract number of 100ns ticks between Jan 1, 1970 and Jan 1, 1600
  
  #if defined(__linux__)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 1164447360 * 100000000;  
  #elif defined(WIN32)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000;  
  #endif

  
  // Convert to seconds since Jan 1, 1970 (so we can use this time with localtime())
  nTimeStampSec.nUint64 = nTimeStamp.nUint64 / 10000000;
  
  // Calculate number of milliseconds (since number of seconds is not a very accurate way to report timestamp)
  nTimeStampMs.nUint64 = nTimeStamp.nUint64 / 10000 - (nTimeStampSec.nUint64 * 1000);
  
  nCurrentTimeSecs = nTimeStampSec.sUint32.nLow;
  ptCurrentDayAndTime = localtime(&nCurrentTimeSecs);
  
  if (ptCurrentDayAndTime) {
    
    DemoLogf("%.2d-%.2d %.2d:%.2d:%.2d.%.3d ", 
      ptCurrentDayAndTime->tm_mon + 1,
      ptCurrentDayAndTime->tm_mday,
      ptCurrentDayAndTime->tm_hour,
      ptCurrentDayAndTime->tm_min, 
      ptCurrentDayAndTime->tm_sec,
      nTimeStampMs.sUint32.nLow
      );
  } else {
    DemoLogf("Error: Invalid timestamp (before Jan 1, 1970)\n");
  }
    
}


/*******************************************************************************
  @func
    Simple state machine (Moore model) to keep track of the Call State
    of an LI. When a call state event occurs, this function is called.
    This function makes a transition in the state machine and stores
    the new call state in g_eCurrentCallState[].
*******************************************************************************/
void DemoCallStateTransition(
  OTX_INT32 nDev,
  DemoCallStateEventT eCallStateEvent // Call State event occurring
)
{
  DemoCallStateT   eOldCallState;
  static const DemoCallStateT s[8] /* no of states */ [5] /* no of events */ = {
// Events=>             DSCE_TIMER   DSCE_PAUSE      DSCE_RING_TONE   DSCE_BUSY_TONE  DSCE_DIAL_TONE
/* States          */
/* DSC_SILENT      */ { DCS_SILENT , DCS_SILENT    , DCS_RING_PEND1 , DCS_BUSY_PEND1, DCS_DIAL       } ,
/* DSC_RING_PEND1  */ { DCS_SILENT , DCS_RING_PEND2, DCS_SILENT     , DCS_SILENT    , DCS_DIAL       } ,
/* DSC_RING_PEND2  */ { DCS_SILENT , DCS_SILENT    , DCS_RING       , DCS_SILENT    , DCS_DIAL       } ,
/* DSC_RING        */ { DCS_SILENT , DCS_RING      , DCS_RING       , DCS_SILENT    , DCS_DIAL       } ,
/* DSC_BUSY_PEND1, */ { DCS_SILENT , DCS_BUSY_PEND2, DCS_SILENT     , DCS_SILENT    , DCS_DIAL       } ,
/* DSC_BUSY_PEND2, */ { DCS_SILENT , DCS_SILENT    , DCS_SILENT     , DCS_BUSY      , DCS_DIAL       } ,
/* DSC_BUSY,       */ { DCS_SILENT , DCS_BUSY      , DCS_SILENT     , DCS_BUSY      , DCS_DIAL       } ,
/* DSC_DIAL        */ { DCS_DIAL   , DCS_SILENT    , DCS_SILENT     , DCS_SILENT    , DCS_DIAL       }
  };

  OsaMutexLock(&g_demoCallStateTransitionCritSec);

  if (nDev > DEMO_POTS_LI_MAX) {
    printf("Error: nDev too large in DemoCallStateTransition\n");
    return;
  }

  eOldCallState = g_eCurrentCallState[nDev];

  g_eCurrentCallState[nDev] = s[ eOldCallState ] [ eCallStateEvent ];

  OsaMutexUnlock(&g_demoCallStateTransitionCritSec);

}


/*******************************************************************************
  @func
    Function to keep track of the call state for each LI

    Runs a separate thread (one for each LI).
*******************************************************************************/
void  *DemoCallStateFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;
  OTX_UINT32 nSourceId;
  OTX_RESULT otxRc;
  char msgStr[256];

  nSourceId = *((OTX_INT32 *) lpParameter);

  
  // Discard any events from event queue (if any)
  do {
    result = OtxDrvGetEventData(g_hLogicDevs.mDualToneDetector[nSourceId].hEventQueue, &eventData); //DemoCheckRc(result);
  } while (result == OTX_S_OK);


  while (1) {

    result = OtxDrvWaitForSingleEvent(
      g_hLogicDevs.mDualToneDetector[nSourceId].hNotificationEvent, 
      5000 // timeout in milliseconds
      );
    
    if (result == OTX_S_OK) {
      //DemoLogf("DualDetector[%d] Dual Tone Event signalled\n", nSourceId);
      do {
        result = OtxDrvGetEventData(g_hLogicDevs.mDualToneDetector[nSourceId].hEventQueue, &eventData); DemoCheckRc(result);
        
        if (result == OTX_S_OK) {
        
          if (eventData.m_eDeviceType != OTX_LDEVICE_TONE_DUAL_DETECTOR) {
            DemoLogf("DualDetector[%d] Error: Unexpected Device Type (0x%X). Thread exiting.\n", nSourceId, eventData.m_eDeviceType);
            break; 
          }
          
          /*******************************************************************************
          
            Dual Tone Coding Matrix 
            
              .         FreqA	 FreqB  FreqC  FreqD  FreqE	 FreqF 
              FreqA    { -1,    1,      2,     4,     7,    11  },
              FreqB    {  1,   -1,      3,     5,     8,    12  },
              FreqC    {  2,    3,     -1,     6,     9,    13  },
              FreqD    {  4,    5,      6,    -1,    10,    14  },
              FreqE    {  7,    8,      9,    10,    -1,    15  },
              FreqF    { 11,   12,     13,    14,    15,    -1  }
              
          *********************************************************************************/
          
          otxRc = OtxDrvEventCode2String(eventData.m_hDevice, eventData.m_nCode, 256, &msgStr[0]);
          if (OTX_SUCCESS(otxRc)) {
            //Event from Dual Tone Detector (our dial, ring, and busy tone detector)
            if (eventData.m_nCode == OTX_DUAL_TONE_DETECTOR_EC_TONE) {
              // 350/440 is dial tone (dualtone index 1)
              // 440/480 is ring tone (dualtone index 3)
              // 480/620 is busy tone (dualtone index 6)
              
              switch (eventData.m_nParam) {
              case 1:
                DemoLogf("DualDetector[%d] detected Dial Tone (%d) \n", eventData.m_nSourceId, eventData.m_nParam);
                DemoCallStateTransition(nSourceId, DCSE_DIAL_TONE);
                break;
              case 3:
                DemoLogf("DualDetector[%d] detected Ring Tone (%d) \n", eventData.m_nSourceId, eventData.m_nParam);
                DemoCallStateTransition(nSourceId, DCSE_RING_TONE);
                break;
              case 6:
                DemoLogf("DualDetector[%d] detected Busy Tone (%d) \n", eventData.m_nSourceId, eventData.m_nParam);
                DemoCallStateTransition(nSourceId, DCSE_BUSY_TONE);
                break;
              default:
                DemoLogf("DualDetector[%d] detected unrecognized tone (%d) \n", eventData.m_nSourceId, eventData.m_nParam);
                break;
              }
              
            } else if (eventData.m_nCode == OTX_DUAL_TONE_DETECTOR_EC_PAUSE) {
              DemoLogf("DualDetector[%d] detected Pause\n", eventData.m_nSourceId);
              DemoCallStateTransition(nSourceId, DCSE_PAUSE);

            } else if (eventData.m_nCode == OTX_DUAL_TONE_DETECTOR_EC_VERBOSE_STATUS) {
              DemoLogf("DualDetector[%d] Verbose Status 0x%.8X (Code: %d, Value: %d)\n", eventData.m_nSourceId, eventData.m_nParam, eventData.m_nParam & 0x0000FFFF, eventData.m_nParam >> 16);

            } else {
              // Unexpected event from this device
              DemoLogf("DualDetector[%d] Unexpected Event: %s \n", eventData.m_nSourceId, msgStr);
            }
          } else {
            DemoLogf("DualDetector[%d] Dual Tone Detector; Invalid Event: %ld \n", eventData.m_nSourceId, eventData.m_nCode);
          }
                    
        }
      } while (result == OTX_S_OK);

    } else if (result == OTX_S_TIMEOUT) {
      //DemoLogf("DualDetector[%d] Dual Tone Timeout\n", nSourceId);

      DemoCallStateTransition(nSourceId, DCSE_TIMER);

    } else {
      DemoLogf("DualDetector[%d] Error: OtxDrvWaitForSingleEvent() returned 0x%X in DemoCallStateFunction(). Thread exiting.\n", nSourceId, result);
      break; 
    }
  }
  
  return 0;
}


/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void  *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;

  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(result);

      if (result == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (result == OTX_S_OK);
  }

  return 0;

}


void DemoPrintEvent(
/*******************************************************************************
  @func
    Prints a Driver Event. If HDLC Data, reads the data.
*******************************************************************************/
  OtxEventDataS *pEventData
)
{
  OTX_RESULT otxRc;
  char msgStr[256];

  DemoPrintTimeStamp(pEventData->m_nTimeStamp);

  switch (pEventData->m_eDeviceType) {

  case OTX_DEVICE_DSP:
    // Non-logical device related event from the DSPs
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      DemoLogf("DSP[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
    } else {
      DemoLogf("DSP[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_TONE_DTMF_DIALER:
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      DemoLogf("DtmfDialler[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
    } else {
      DemoLogf("DtmfDialler[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_TONE_DTMF_DETECTOR:
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      // Event from Dtmf Detector
      if (pEventData->m_nCode == OTX_DTMF_DETECTOR_EC_DIGIT) {
        DemoLogf("DtmfDetector[%d] detected DTMF digit %d \n", pEventData->m_nSourceId, pEventData->m_nParam);
      } else {
        // Unexpected event from this device
        DemoLogf("DtmfDetector[%d] Unexpected Event: %s \n", pEventData->m_nSourceId, msgStr);
      }
    } else {
      DemoLogf("DtmfDetector[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;


  case OTX_DEVICE_LI_POTS:
    // Event from Alis Detector
    switch (pEventData->m_nCode) {
    case OTX_POTS_WAKE_UP:
      DemoLogf("Li[%d] Wake up from Power-down mode (>18V at Tip/Ring)\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_RING_SIGNAL_ON:
      DemoLogf("Li[%d] Ring pulse on\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_RING_SIGNAL_OFF:
      DemoLogf("Li[%d] Ring pulse off\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_RING_SIGNAL_DETECTED:
      DemoLogf("Li[%d] Ring signal detected\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_INVALID_RING_SIGNAL:
      DemoLogf("Li[%d] Invalid ring signal received\n", pEventData->m_nSourceId);
      break;
    case OTX_POTS_CALLER_ID:
      DemoLogf("Li[%d] Caller ID received\n", pEventData->m_nSourceId);
      break;
    default:
      // Unexpected event from this device
      DemoLogf("Li[%d] Unexpected Event (%d)\n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }

}



OTX_INT32 DemoProcessUserCommand(char commandChar)
/*******************************************************************************
  @func
    Processes User Commands
*******************************************************************************/
{
  OTX_RESULT        result;
  static OTX_INT32  nCurrentPotsLine = 0;
  static OTX_HANDLE hCurrentPotsLine;
  hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd   Synopsis\n");
    DemoLogf("===   ===========================================================\n");
    DemoLogf(" ?    Display Help text\n");
    DemoLogf(" h    Ditto\n");
    DemoLogf(" q    Quit\n");
    DemoLogf(" +    Select the next POTS line as active POTS Line (target of\n");
    DemoLogf("      subsequent commands). Connect Handset 0 to the active Line\n");
    DemoLogf(" -    Select the previous POTS line as the active POTS Line (target\n");
    DemoLogf("      of subsequent commands). Connect Handset 0 to the active Line\n");
    DemoLogf(" 0..9 Dial Digit 0 .. 9 on at a time\n");
    DemoLogf(" f    Set active POTS Line Off-Hook\n");
    DemoLogf(" n    Set active POTS Line On-Hook\n");
    DemoLogf(" s    Get current call state (silent/busy/ring/dial) for active POTS Line\n");
    DemoLogf(" d    Call DTMF digits 102 on the active POTS Line\n");
    DemoLogf(" D    Dial user specified DTMF digits on the active POTS Line\n");
    DemoLogf(" t    Turn Sweeping Sine Wave On/Off on the active POTS Line\n");
    DemoLogf(" m    Display the Operation Mode of the active Line Interface Device\n");
    DemoLogf(" M    Set the Operation Mode of the active Line Interface Device\n");
    DemoLogf(" e/E  Enable/Disable interupts from the active POTS Line\n");
    DemoLogf(" r    Read from an 8-bit register in Alis Pots Line\n");
    DemoLogf(" w    Write to an 8-bit register in Alis Pots Line\n");
    break;

  case 'e':
    DemoLogf("Enable Li #%d \n", nCurrentPotsLine);
    result = OtxDrvEnable(g_hPhysDevs.hLi[nCurrentPotsLine], DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case 'E':
    DemoLogf("Disable Li #%d \n", nCurrentPotsLine);
    result = OtxDrvDisable(g_hPhysDevs.hLi[nCurrentPotsLine], DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case '-':
    {
      // Select the previous POTS line as the active line. Cross-Connect handset 0 to that line ...
      OTX_INT32 nPrevPotsLine = nCurrentPotsLine;

      if ( nCurrentPotsLine == 0 ) {
        nCurrentPotsLine = DEMO_POTS_LI_MAX - 1;
      } else {
        nCurrentPotsLine--;
      }

      // Disconnect Codec#0 from the old active line
      result = OtxDrvDisconnectPipe(g_hPipes.hDspCodec[nPrevPotsLine], OTX_PIPE_OUTPUT); DemoCheckRc(result);
      result = OtxDrvDisconnectPipe(g_hPipes.hCodecDsp[nPrevPotsLine], OTX_PIPE_INPUT);  DemoCheckRc(result);

      // Reconnect Codec #0 to the current Active POTS Line
      result = OtxDrvConnectPipe(g_hPipes.hDspCodec[nCurrentPotsLine], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);
      result = OtxDrvConnectPipe(g_hPipes.hCodecDsp[nCurrentPotsLine], OTX_PIPE_INPUT,  g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);

      hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
      DemoLogf("POTS Line # %d Selected as the Active Line\n", nCurrentPotsLine);
    }
    break;

  case '+':
    {
      // Select the next POTS line as the active line. Cross-Connect handset 0 to that line ...
      OTX_INT32 nPrevPotsLine = nCurrentPotsLine;

      if ( nCurrentPotsLine == (DEMO_POTS_LI_MAX - 1) ) {
        nCurrentPotsLine = 0;
      } else {
        nCurrentPotsLine++;
      }

      // Disconnect Codec#0 from the previously  active line
      result = OtxDrvDisconnectPipe(g_hPipes.hDspCodec[nPrevPotsLine], OTX_PIPE_OUTPUT); DemoCheckRc(result);
      result = OtxDrvDisconnectPipe(g_hPipes.hCodecDsp[nPrevPotsLine], OTX_PIPE_INPUT);  DemoCheckRc(result);

      // Reconnect Codec #0 to the current Active POTS Line
      result = OtxDrvConnectPipe(g_hPipes.hDspCodec[nCurrentPotsLine], OTX_PIPE_OUTPUT, g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);
      result = OtxDrvConnectPipe(g_hPipes.hCodecDsp[nCurrentPotsLine], OTX_PIPE_INPUT,  g_hPhysDevs.hCodec[0], -1, NULL, 0); DemoCheckRc(result);

      hCurrentPotsLine = g_hPhysDevs.hLi[nCurrentPotsLine];
      DemoLogf("POTS Line # %d Selected as the Active Line\n", nCurrentPotsLine);
    }
    break;

  case 'f':
    // Set Active POTS Line Off-Hook ...
    DemoLogf("POTS Line #%d Off-hook\n", nCurrentPotsLine);
    result = OtxPotsOffHook(hCurrentPotsLine); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;


  case 'n':
    // Set Active POTS Line On-Hook ...
    DemoLogf("POTS Line #%d On-hook\n", nCurrentPotsLine);
    result = OtxPotsOnHook(hCurrentPotsLine); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case 'l':
    // Flash Hook
    DemoLogf("POTS Line #%d Hook Flash\n", nCurrentPotsLine);
    result = OtxPotsHookFlash(hCurrentPotsLine, 200, OTX_TASK_SYNC, 5000L); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;

  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
    { // Dial Digits one at a time ...
      char digitStr[1];
      OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialler[nCurrentPotsLine];

      DemoLogf("Dialing digit '%d'\n", commandChar);

      digitStr[0] = commandChar;

      result = OtxToneDtmfDial(hHandle, (unsigned char *)digitStr, 1, DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);

      DemoLogf("Done.\n");
    }
    break;

  case 's':
    {
      // Display the current call state for the current POTS line
      OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialler[nCurrentPotsLine];
      char *pszCallStatus;

      switch (g_eCurrentCallState[nCurrentPotsLine]) {
      case DCS_SILENT:
        pszCallStatus = "Silent";
        break;
      case DCS_RING_PEND1:
        pszCallStatus = "Ringing pending (waiting for pause)";
        break;
      case DCS_RING_PEND2:
        pszCallStatus = "Ringing pending (waiting for second ring tone)";
        break;
      case DCS_RING:
        pszCallStatus = "Ringing";
        break;
      case DCS_BUSY_PEND1:
        pszCallStatus = "Busy pending (waiting for pause)";
        break;
      case DCS_BUSY_PEND2:
        pszCallStatus = "Busy pending (waiting for second busy tone)";
        break;
      case DCS_BUSY:
        pszCallStatus = "Busy";
        break;
      case DCS_DIAL:
        pszCallStatus = "Dial";
        break;
      default:
        pszCallStatus = "Unknown";
        break;
      }


      DemoLogf("Call state for POTS Line #%d = %s\n", nCurrentPotsLine, pszCallStatus);

    }
    break;

  case 'd':
    {
      // Dial a hard-coded number ...
      OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialler[nCurrentPotsLine];

      DemoLogf("Sending digits on POTS Line #%d: '%s'", nCurrentPotsLine, g_sqNumber0);

      // Start Dialing DTMF number, Non-Blocking call, will receive Event when completed
      result = OtxToneDtmfDial(hHandle, (unsigned char *)g_sqNumber0, strlen(g_sqNumber0), DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);

      DemoLogf("Done.\n");
    }
    break;

  case 'D':
    {
      // Dial a user specified number ...
      char strBuf[32];
      OTX_HANDLE hHandle = g_hLogicDevs.hDtmfDialler[nCurrentPotsLine];

      DemoLogf("Phone number to Dial: ");
      OsaConsoleGetStr(strBuf);
      DemoLogf("POTS Line #%d Dialling number '%s'", nCurrentPotsLine, strBuf);

      // Start Dialing DTMF number, Non-Blocking call, will receive Event when completed
      result = OtxToneDtmfDial(hHandle, (unsigned char *)strBuf, strlen(strBuf), DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);

      DemoLogf("Done.\n");
    }
    break;

  case 't':
    { // Toggle Sweeping sine-wave on and off ...
      OTX_HANDLE hHandle;
      OTX_BOOL   *pbOn;

      pbOn = &g_bSineWaveOn[nCurrentPotsLine];
      hHandle = g_hLogicDevs.hToneGenerator[nCurrentPotsLine];

      if (!*pbOn) {
        // Turn it On
        // Turn Sweeping Sine Wawe On using the Tone Effects Generator
        DemoLogf("Turn on Sweeping Sine Wave On on Line #%d: \n", nCurrentPotsLine);

        // Send DTMF Digits, Blocking call, wait until completed
        result = OtxDrvEnable(hHandle, 0L, 5000L); DemoCheckRc(result);
        *pbOn = OTX_TRUE;

      } else {
        // Turn it Off
        // Turn Sweeping Sine Wawe On using the Tone Effects Generator
        DemoLogf("Turn off Sweeping Sine Wave On on Line #%d: \n", nCurrentPotsLine);

        // Send DTMF Digits, Blocking call, wait until completed
        result = OtxDrvDisable(hHandle, 0L, 5000L); DemoCheckRc(result);

        *pbOn = OTX_FALSE;
      }
    }
    break;

  case 'm':
    {
      // Read the operation mode of the Active LI Device ...
      OtxPotsLiModeE eLiMode;

      result = OtxPotsGetLiMode(hCurrentPotsLine, &eLiMode);
      DemoLogf("POTS Line #%d is in mode 0x%.4X\n", nCurrentPotsLine, eLiMode);

    }
    break;

  case 'M':
    {
      // Set the operation mode of the Active LI Device ...
      OtxPotsLiModeE eLiMode;
      char strBuf[32];

      DemoLogf("Set POTS Line #%d Mode:\n");
      DemoLogf("Sleep         :  0\n");
      DemoLogf("Ringing       :  1\n");
      DemoLogf("Conversation  :  2\n");
      DemoLogf("Pulse Dialing :  3\n");
      DemoLogf("--> ");
      OsaConsoleGetStr(strBuf);
      eLiMode = (OtxPotsLiModeE) strtoul(strBuf, NULL, 0);

      result = OtxPotsSetLiMode(hCurrentPotsLine, eLiMode);
      DemoLogf("POTS Line #%d is set to mode %d\n", nCurrentPotsLine, eLiMode);
    }
    break;

  case 'p':
    // Pulse dial g_sqNumber0
    DemoLogf("POTS Line #%d Pulse Dialing '%s'\n", nCurrentPotsLine, g_sqNumber0);
    result = OtxPotsPulseDial(hCurrentPotsLine, 40, 60, 500, (unsigned char *)g_sqNumber0, strlen(g_sqNumber0), DEMO_TASK_REF, OTX_TIME_INFINITY); DemoCheckRc(result);
    DemoLogf("Done.\n");
    break;


#if 1 // For debugging
  case 'L':
    DemoLogf("ALIS#%d in Loop mode (AIC%d[%d])\n", nCurrentPotsLine, nCurrentPotsLine/8, nCurrentPotsLine%8);
    OtxDrvDbgDump(g_hPhysDevs.hLiController[nCurrentPotsLine/8], nCurrentPotsLine%8);
    break;


  case 'c':
    {
      OTX_UINT8 highWay;
      OTX_UINT8 channel;
      OTX_UINT8 byteVal;
      char wValStr[12];
      
      DemoLogf("Set Constant Byte on Channel\n");
      
      DemoLogf("Highway: ");
      OsaConsoleGetStr(wValStr);
      highWay = (OTX_UINT8) strtoul(wValStr, NULL, 0);
      
      DemoLogf("Channel: ");
      OsaConsoleGetStr(wValStr);
      channel = (OTX_UINT8) strtoul(wValStr, NULL, 0);
      
      DemoLogf("Constant Byte: ");
      OsaConsoleGetStr(wValStr);
      byteVal = (OTX_UINT8) strtoul(wValStr, NULL, 0);
      
      result = OtxBrdTssConstByte(g_hPhysDevs.hTss, highWay, channel, byteVal); DemoCheckRc(result);

      DemoLogf("Done.\n");
    }
    break;

  case 'C':
    {
      OTX_UINT8 highWay;
      OTX_UINT8 channel;
      OTX_UINT8 byteVal;
      char wValStr[12];
      
      DemoLogf("Set Constant Byte on All the Channels of AIC \n");
      
      DemoLogf("Highway: ");
      OsaConsoleGetStr(wValStr);
      highWay = (OTX_UINT8) strtoul(wValStr, NULL, 0);
      
/*      DemoLogf("Channel: ");
      OsaConsoleGetStr(wValStr);
      channel = (OTX_UINT8) strtoul(wValStr, NULL, 0);
*/      
      DemoLogf("Constant Byte: ");
      OsaConsoleGetStr(wValStr);
      byteVal = (OTX_UINT8) strtoul(wValStr, NULL, 0);
      
      for (channel = 2; channel < 18; channel++) {
        result = OtxBrdTssConstByte(g_hPhysDevs.hTss, highWay, channel, byteVal); DemoCheckRc(result);
      }

      DemoLogf("Done.\n");
      
    }
    break;
#endif
	  
  case 'r':
		{
      OTX_UINT32 readVal;
      char strBuf[12];
      static OTX_UINT32 nAddress = 0;

      DemoLogf("Read from an 8-bit register in Alis Pots Line%d \n", nCurrentPotsLine);
      DemoLogf("Which Address (0x%X): ", nAddress);

      OsaConsoleGetStr(strBuf); 
      if (*strBuf) {
        nAddress = strtoul(strBuf, NULL, 0);
      }

      result = OtxDrvReadReg8(g_hPhysDevs.hLi[nCurrentPotsLine], nAddress, &readVal); DemoCheckRc(result);

      DemoLogf("Alis Pots Line%d Address 0x%X = %.2X\n", nCurrentPotsLine, nAddress, (OTX_UINT8)readVal);
    }
    break;

  case 'w':
    {
      char strBuf[12];
      static OTX_UINT32 nAddress = 0;
      static OTX_UINT32 nValue = 0;

      DemoLogf("Write to an 8-bit register in Alis Pots Line%d \n", nCurrentPotsLine);
      DemoLogf("Which Address (0x%X): ", nAddress);
      OsaConsoleGetStr(strBuf);  
      if (*strBuf) {
        nAddress = strtoul(strBuf, NULL, 0);
      }
      DemoLogf("Which Value (0x%X): ", nValue);
      OsaConsoleGetStr(strBuf);   
      if (*strBuf) {
        nValue = strtoul(strBuf, NULL, 0);
      }

      result = OtxDrvWriteReg8(g_hPhysDevs.hLi[nCurrentPotsLine], nAddress, nValue); DemoCheckRc(result);

      DemoLogf("Wrote %.2X to Alis Pots Line%d Address 0x%X\n", (OTX_UINT8)nValue, nCurrentPotsLine, nAddress);
    }
    break;

  case 'q':
  case 'Q':
    // Quit ...
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}



/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->logFileName  = DEMO_DEFAULT_LOG_FILE;
  envP->law          = OTX_DATA_FORMAT_COMPAND8_U_LAW;
  envP->boardNo      = DEMO_DEFAULT_BOARD_NO;
  envP->nSetId       = OTX_POTS_CFG_SET_USA;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    OsaStdStrUpr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'L') {   // Log file
        envP->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'B') {   // Board number
        envP->boardNo = strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'C') {   // Companding Law
        if (*(argv[i]+2) == 'A') {   // A-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_A_LAW;
        } else if (*(argv[i]+2) == 'U') {   // U-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_U_LAW;
        } else {
          DemoLogf("Error: Invalid companding law command line switch %s\n", argv[i]);
          DemoHelpPanel();
					OsaConsoleExit();
          exit(1);
        }
      } else if (*(argv[i]+1) == 'S') {   // Configuration Set
        if (*(argv[i]+2) == 'U') {          // USA
          envP->nSetId = OTX_POTS_CFG_SET_USA;
        } else if (*(argv[i]+2) == 'G') {   // Germany
          envP->nSetId = OTX_POTS_CFG_SET_GERMANY;
        } else if (*(argv[i]+2) == 'E') {   // UK
          envP->nSetId = OTX_POTS_CFG_SET_UK;
        } else if (*(argv[i]+2) == 'F') {   // France
          envP->nSetId = OTX_POTS_CFG_SET_FRANCE;
        } else if (*(argv[i]+2) == 'A') {   // Australia
          envP->nSetId = OTX_POTS_CFG_SET_AUSTRALIA;
        } else if (*(argv[i]+2) == 'J') {   // Japan
          envP->nSetId = OTX_POTS_CFG_SET_JAPAN;
        } else {
          DemoLogf("Error: Unsupported country specific configuration set %s\n", argv[i]);
          DemoHelpPanel();
					OsaConsoleExit();
          exit(1);
        }
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
				exit(1);
      }
    } else {
      DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
      DemoHelpPanel();
      OsaConsoleExit();
			exit(1);
    }
  }

}


static void DemoHelpPanel()
/*******************************************************************************
  @func
    Displays a help for the command line.
*******************************************************************************/
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /B<boardNo>  : Board number (Default is %d)\n", DEMO_DEFAULT_BOARD_NO);
  DemoLogf(" /CA          : A-law\n");
  DemoLogf(" /CU          : u-law\n");
  DemoLogf(" /SU          : Li Configuration for USA (Default)\n");
  DemoLogf(" /SG          : Li Configuration for Germany\n");
  DemoLogf(" /SE          : Li Configuration for United Kingdom\n");
  DemoLogf(" /SF          : Li Configuration for France\n");
  DemoLogf(" /SA          : Li Configuration for Australia\n");
  DemoLogf(" /SJ          : Li Configuration for Japan\n");
}


static void DemoLogf(char *formatStr, ...)
/*******************************************************************************
  @func
    Displays a message on the screen and prints it to the logfile.
*******************************************************************************/
{
  va_list argList;

  OsaMutexLock(&g_demoLogfCritSec);

  va_start(argList, formatStr);

  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) vfprintf(g_pDemoLogFile,   formatStr, argList);

  va_end(argList);

  OsaMutexUnlock(&g_demoLogfCritSec);
}



/*******************************************************************************
*******************************************************************************/
int main(int argc, char *argv[])
{
  OTX_INT32  i;
  OTX_INT32  exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char       commandChar;

	OsaConsoleInit();

  OsaMutexInit(&g_demoLogfCritSec);
  OsaMutexInit(&g_demoCallStateTransitionCritSec);

  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    exit(-1);
  }

  DemoLogf("%s, %s, Odin TeleSystems Inc.\n", argv[0], __DATE__);
  DemoLogf("Arni-16-PCI, Vidar-5x4-ASM Demo Program.\n");

  DemoLogf("Log file is %s\n", g_demoEnv.logFileName);
  DemoLogf("Board number %d\n", g_demoEnv.boardNo);


  // Query Library revision
  {
    char szVersionInfo[255];
    OTX_BOOL bIsStaticLinkedLibrary;
    char szLibFileName[64];
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_REV_STR, szVersionInfo, 255); DemoCheckRc(result);
    result = OtxDrvGetAttributeValueBool(OTX_LIB_HANDLE, OTX_ATTR_LIB_TYPE_STATIC, &bIsStaticLinkedLibrary); DemoCheckRc(result);
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_FILE_NAME, szLibFileName, 64); DemoCheckRc(result);
    
    DemoLogf("Library version %s (filename %s, %s linked)\n", szVersionInfo, szLibFileName, bIsStaticLinkedLibrary ? "static":"dynamic");
  }


  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); 
  switch (OTX_RESULT_CODE(result)) {
  case OTX_RES_HANDLE:
    DemoLogf("Error: Could not connect to the OTX library.\nPossible cause: OTX Driver not loaded\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  case OTX_RES_NOT_SUPPORTED:
    DemoLogf("Error: Mismatch between OTX Driver revision and OTX Library revision.\nRevisions must match.\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  default:
    DemoCheckRc(result);
    break;
  }

  // Create Event Queues for Driver --> Application Communications
  DemoCreateEventQueue();

  // Connect (get handles) to the physical devices on the Arni-16-PCI board
  DemoOpenPhysicalDevices();

  // Initialized the Connected Logical Devices
  DemoInitializePhysicalDevices();

  // Enable Physical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoEnablePhysicalDevices();

  // Create and get handles to the Logical devices needed by this application
  DemoCreateLogicalDevices();

  // Create and get handles to the Pipe devices needed by this application
  // Connect Pipes to Physical and Logical Devices
  DemoConnectDevices();

  // Initialized the Connected Logical Devices
  DemoInitializeLogicalDevices();

  // Enable Logical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoEnableLogicalDevices();

  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....

  OsaThreadStart(DemoIdleFunction);

  OsaThreadSleep(200);

  for (i = 0; i < 1 /*DEMO_POTS_LI_MAX*/; i++) {
    g_hLogicDevs.mDualToneDetector[i].nSourceId = i;
    OsaThreadStartEx(DemoCallStateFunction, &g_hLogicDevs.mDualToneDetector[i].nSourceId);
  }

  DemoLogf("\nPress 'h' for help, 'q' to quit ...\n");

  while (!exitFlag) {
    commandChar = getch();
		exitFlag = DemoProcessUserCommand(commandChar);
  }

  // Delete allocated resources
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

