/* $Workfile: T2p5x4Fsk.c $       $Revision: 3 $      $Modtime: 8/01/02 3:35p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    T2P5X4FSK.C

    A simple example console application demonstrating the use of Raw Data Sender
    Silence Detector and the FSK Detector devices with the THOR-2-PCI board and 
    Vidar-5x4-ASM Daughter Board.

    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    The Codec (HS0) is connected with two pipes to DSP 0.

    With keystroke command 'p' you can playback a prerecorded FSK file from the
    disk. The sound will play in HS0. The sound will also be looped back to
    the DSP where a FSK detector and a Silence detector is analyzing the data
    which is played from the file.

    A file called FskAlaw.Raw is supplied. When this file is played the following
    FSK sequence (30 bytes) should be detected:

    "\x80\x1B\x11\x01\x01\x01\x08\x31\x30\x31\x35\x31\x31\x35\x30\x02\x0C\x30\x31\x32\x37\x33\x2D\x35\x30\x30\x35\x35\x36\x4E"

    A command line switch specifies whether U-law or A-law should be used for the
    recording/playback

    Press 'h' or '?' for help. 'q' to quit.

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>


#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
#include "OtxErr.h"
#include "OtxBrd.h"
#include "OtxDef.h"
#include "OtxType.h"
#include "OtxDrv.h"
#include "OtxDsp.h"
#include "OtxRaw.h"
#include "OtxT1E1.h"
#include "OtxData.h"
#include "OtxTone.h"

#define DEMO_TASK_REF 1234               // Task Reference for a non-blocking call


#define DEMO_RAW_TX_BUF_SIZE 1024

#define DEMO_DEFAULT_PLAYBACK_FILE "FskAlaw.raw"
#define DEMO_DEFAULT_LOG_FILE "T2p5x4Fsk.log"
#define DEMO_DEFAULT_BOARD_NO 0


typedef struct DemoEnvT {
  char           *logFileName;
  OtxDataFormatE  eLaw;
  int             boardNo;
  OtxDspSpmE      eProgramModule;      
} DemoEnvT;


/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoPrintEvent(IN OtxEventDataS *pEventData);
static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT otxRc);
static void DemoLogf(char *formatStr, ...);
static void DemoHelpPanel(void);
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);


/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue  = 0;
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;



struct {
  OTX_HANDLE hThorController0;
  OTX_HANDLE hLi[2]; // For dummy connections
  OTX_HANDLE hTss;
  OTX_HANDLE hVidarAsmController0;
  OTX_HANDLE hDsp;
  OTX_HANDLE hCodec;
} g_hPhysDevs;


struct {
  OTX_HANDLE hDspCodec;
  OTX_HANDLE hCodecDsp;
} g_hPipes;

struct sRawDataSender {
  OTX_HANDLE hHandle;
  OTX_HANDLE hEventQueue;
  OTX_EVENT  hNotificationEvent;

  OTX_UINT32 nThreadId;
  OTX_HANDLE hThread;

  OTX_UINT8   sqTxBuf[DEMO_RAW_TX_BUF_SIZE];
  FILE       *pTxFile;
  OTX_INT32   cbTxBuf;
  OTX_INT32   nTaskRef;

};

struct sRawDataReceiver {
  OTX_HANDLE hHandle;
  OTX_HANDLE hEventQueue;
  OTX_EVENT  hNotificationEvent;

  OTX_UINT32 nThreadId;
  OTX_HANDLE hThread;

  FILE       *pRxFile;
  clock_t     nMsDuration;
};

struct sSilenceDetector {
  OTX_HANDLE hHandle;
};

struct sFskDetector {
  OTX_HANDLE hHandle;
};



struct {
  struct sRawDataSender mRawDataSender;
  struct sRawDataReceiver mRawDataReceiver;
  struct sSilenceDetector mSilenceDetector;
  struct sFskDetector mFskDetector;
} g_hLogicDevs;


/*******************************************************************************
*******************************************************************************/


static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  short c;
  if ( OTX_FAILURE(result) ) {

    if (hDevice == OTX_INVALID_HANDLE_VALUE) {
      DemoLogf("ERROR(%#lX) \n", result);
    } else {
      OTX_RESULT otxRc;
      char msgStr[256];
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      OTX_UINT32      eDeviceType = 999;    // Device type for hDevice 
      OTX_UINT32      nDeviceNumber = 999;  // Device number for hDevice
      
      // Get the Device Type for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_TYPE, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_TYPE, &eDeviceType); 
      } 
      
      // Get the Device Number for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_NO, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDeviceNumber); 
      }
      
      // Convert the result code to a more descriptive string
      otxRc = OtxDrvResultCode2String(hDevice, result, 256, &msgStr[0]);
      if (OTX_SUCCESS(otxRc)) {
        DemoLogf("ERROR(%#lX) Dev(%X,%d,%d): %s \n", result, hDevice, (int) eDeviceType, (int)nDeviceNumber, msgStr);
      } else {
        DemoLogf("ERROR(%#lX) Dev(%X,%d,%d) Cannot translate this result code \n", result, hDevice, (int) eDeviceType, (int)nDeviceNumber);
      }
    }

    DemoLogf("Do you want to exit (Y/N) ");
    c = getch();
    if (c == 'y' || c == 'Y') {
      // Clean up
      OtxDrvReleaseLib();
      OsaConsoleExit();
			exit(1);
    }

  }
}



/*******************************************************************************
  @func
    Raw Data Receiver Thread Function 
*******************************************************************************/
void *DemoRawDataReceiver(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;
  OTX_RESULT otxRc;
  OTX_UINT8  buf[2048];
  OTX_UINT32 nFrameLen;
  OTX_UINT32 nMaxFrameLen = 0;
  OTX_UINT32 nEventCount = 0;
  OTX_UINT32 nReadDataCount = 0;
  char msgStr[256];
  clock_t startTime, stopTime;

  startTime = clock();

  result = OtxDrvEnable(g_hLogicDevs.mRawDataReceiver.hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hHandle, result);


  while (clock() < (startTime + g_hLogicDevs.mRawDataReceiver.nMsDuration)) {
    OtxDrvWaitForSingleEvent(g_hLogicDevs.mRawDataReceiver.hNotificationEvent, OTX_TIME_INFINITY);
    do {
      result = OtxDrvGetEventData(g_hLogicDevs.mRawDataReceiver.hEventQueue, &eventData); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hEventQueue, result);

      nEventCount++;

      if (result == OTX_S_OK) {
        // We have a driver event
        switch (eventData.m_eDeviceType) {
        case OTX_LDEVICE_DATA_RAW_RECEIVER:
          // Event from Raw Data Receiver 

          switch (eventData.m_nCode) {
          case OTX_RAWDATA_RECEIVER_EC_DATA_AVAILABLE:
            {
              OTX_BOOL bFrameEnd;
              otxRc = OtxRawDataReadData(eventData.m_hDevice, buf, 2048, &nFrameLen, &bFrameEnd);
              while (otxRc == OTX_S_OK) {

                nReadDataCount++;
                
                if (g_hLogicDevs.mRawDataReceiver.pRxFile) {
                  int cbWritten;

                  if (nFrameLen > nMaxFrameLen) {
                    nMaxFrameLen = nFrameLen;
                  }

                  cbWritten = fwrite(buf, sizeof(OTX_UINT8), nFrameLen, g_hLogicDevs.mRawDataReceiver.pRxFile);

                  if (cbWritten != (int)nFrameLen) {
                    DemoLogf("RawDataReceiver[%d] Error: Could not write data to file.  (nFrameLen=%d, cbWritten=%d) \n", eventData.m_nSourceId, nFrameLen, cbWritten);
                  }
                  
                  if (bFrameEnd) {
                    DemoLogf("RawDataSender[%d] Frame End occurred (unexpectedly). Data reception complete. Closing data file. \n", eventData.m_nSourceId);
                    fclose(g_hLogicDevs.mRawDataReceiver.pRxFile);
                    g_hLogicDevs.mRawDataReceiver.pRxFile = NULL;
                    goto END;
                  }
                }
                
                otxRc = OtxRawDataReadData(eventData.m_hDevice, buf, 2048, &nFrameLen, &bFrameEnd);
              }
              if (otxRc != OTX_S_FALSE) {
                DemoLogf("RawDataReceiver[%d] OtxRawDataReadData() was unsuccessful.\n", eventData.m_nSourceId);
                DemoCheckRc(eventData.m_hDevice, otxRc);
              }

            }
            break;
          default:
            otxRc = OtxDrvEventCode2String(eventData.m_hDevice, eventData.m_nCode, 256, &msgStr[0]);
            // Unexpected event from this device
            if (OTX_SUCCESS(otxRc)) {
              DemoLogf("RawDataReceiver[%d] Event: %s \n", eventData.m_nSourceId, msgStr);
            } else {
              DemoLogf("RawDataReceiver[%d] Invalid Event: %ld \n", eventData.m_nSourceId, eventData.m_nCode);
            }
            break;
          }
          break;
          default:
            // Unhandled event
            DemoLogf("DemoRawDataSender: Event from an unknown object 0x%X\n", eventData.m_hDevice);
            break;
        }
        
      }
    } while (result == OTX_S_OK);

    OsaThreadSleep(256); // Give the driver some time to fill up more data for us to fetch (256ms fills 2048 bytes in a 64kbit/s pipe)

  }

END:
  
  stopTime = clock();

  result = OtxDrvDisable(g_hLogicDevs.mRawDataReceiver.hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hHandle, result);

  if (g_hLogicDevs.mRawDataReceiver.pRxFile) {
    fclose(g_hLogicDevs.mRawDataReceiver.pRxFile);
    g_hLogicDevs.mRawDataReceiver.pRxFile = NULL;
  }

  DemoLogf("DemoRawDataReceiver: nEventCount=%d, nReadDataCount=%d, nMaxFrameLen=%d\n", nEventCount, nReadDataCount, nMaxFrameLen);
  DemoLogf("DemoRawDataReceiver: Done (time=%2.1f seconds).\n", (double)(stopTime - startTime) / CLOCKS_PER_SEC);
  
  return 0;

}


/*******************************************************************************
  @func
    Raw Data Sender Thread Function 
*******************************************************************************/
void  *DemoRawDataSender(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;
  OTX_RESULT otxRc;
  char msgStr[256];
  clock_t startTime, stopTime;

  // Discard any events from  event queue (if any)
  do {
    result = OtxDrvGetEventData(g_hLogicDevs.mRawDataSender.hEventQueue, &eventData); 
  } while (result == OTX_S_OK);
 
  // Flush (empty) the queue of raw data octets waiting to be transmitted (if any).
  result = OtxRawDataSenderFlush(g_hLogicDevs.mRawDataSender.hHandle); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);

  startTime = clock();
  g_hLogicDevs.mRawDataSender.nTaskRef = 1;

  // Stuff the Raw Data Sender device full with data before we start to wait for OTX_RAWDATA_SENDER_EC_BUF_DEPLETED events
  do {
    g_hLogicDevs.mRawDataSender.cbTxBuf = fread(g_hLogicDevs.mRawDataSender.sqTxBuf, sizeof(OTX_UINT8), DEMO_RAW_TX_BUF_SIZE, g_hLogicDevs.mRawDataSender.pTxFile);
  
    otxRc = OtxRawDataSendData(g_hLogicDevs.mRawDataSender.hHandle, g_hLogicDevs.mRawDataSender.sqTxBuf, g_hLogicDevs.mRawDataSender.cbTxBuf, g_hLogicDevs.mRawDataSender.nTaskRef, OTX_TIME_INFINITY);

    if (otxRc != OTX_E_BUSY) {
      g_hLogicDevs.mRawDataSender.nTaskRef++;
    }

  } while (g_hLogicDevs.mRawDataSender.cbTxBuf && (otxRc != OTX_E_BUSY));

  
  if (g_hLogicDevs.mRawDataSender.cbTxBuf) {
    DemoLogf("DemoRawDataSender: Waiting for OTX_RAWDATA_SENDER_EC_BUF_DEPLETED events to complete the transmission...\n");
  } else {
    DemoLogf("DemoRawDataSender: The entire file fit in the driver buffer. Closing file\n");
    fclose(g_hLogicDevs.mRawDataSender.pTxFile);
    g_hLogicDevs.mRawDataSender.pTxFile = NULL;
  }
  
  
  // Wait for events from the Raw Data Sender
  while (1) {
    OtxDrvWaitForSingleEvent(g_hLogicDevs.mRawDataSender.hNotificationEvent, 5);
    do {
      result = OtxDrvGetEventData(g_hLogicDevs.mRawDataSender.hEventQueue, &eventData); //DemoCheckRc(g_hLogicDevs.mRawDataSender.hEventQueue, result);

      if (result == OTX_S_OK) {
        // We have a driver event
        switch (eventData.m_eDeviceType) {
        case OTX_LDEVICE_DATA_RAW_SENDER:
          // Event from Raw Data Sender 

          switch (eventData.m_nCode) {
          case OTX_RAWDATA_SENDER_EC_BUF_DEPLETED:
            {
              OTX_RESULT otxRc;

              DemoLogf("Buffer sent (eventData.m_nRequestId = %d)\n", eventData.m_nRequestId);
              
              if (g_hLogicDevs.mRawDataSender.pTxFile) {
                while (g_hLogicDevs.mRawDataSender.cbTxBuf) {
                  otxRc = OtxRawDataSendData(g_hLogicDevs.mRawDataSender.hHandle, g_hLogicDevs.mRawDataSender.sqTxBuf, g_hLogicDevs.mRawDataSender.cbTxBuf, g_hLogicDevs.mRawDataSender.nTaskRef, OTX_TIME_INFINITY);

                  if (otxRc == OTX_E_BUSY) {
                    break;
                  } else {
                    g_hLogicDevs.mRawDataSender.nTaskRef++;
                  }

                  g_hLogicDevs.mRawDataSender.cbTxBuf = fread(g_hLogicDevs.mRawDataSender.sqTxBuf, sizeof(OTX_UINT8), DEMO_RAW_TX_BUF_SIZE, g_hLogicDevs.mRawDataSender.pTxFile);
                  
                }
                
                if (!g_hLogicDevs.mRawDataSender.cbTxBuf) {
                  DemoLogf("DemoRawDataSender: EOF reached. Closing file\n");
                  fclose(g_hLogicDevs.mRawDataSender.pTxFile);
                  g_hLogicDevs.mRawDataSender.pTxFile = NULL;
                }
              }
            }
            break;
          
          case OTX_RAWDATA_SENDER_EC_COMPLETED:
            DemoLogf("DemoRawDataSender: Raw Data transmission ended.\n");
            goto END;
            break;
            
          default:
            otxRc = OtxDrvEventCode2String(eventData.m_hDevice, eventData.m_nCode, 256, &msgStr[0]);
            // Unexpected event from this device
            if (OTX_SUCCESS(otxRc)) {
              DemoLogf("DemoRawDataSender: Unexpected Event: %s \n", msgStr);
            } else {
              DemoLogf("DemoRawDataSender: Invalid Event: %ld \n", eventData.m_nCode);
            }
            break;
          }
          break;
          default:
            // Unhandled event
            DemoLogf("DemoRawDataSender: Event from an unknown object 0x%X\n", eventData.m_hDevice);
            break;
        }
        
      }
    } while (result == OTX_S_OK);

#if 1 // Silence check
    {
      OTX_BOOL  bSilence;      
      
      result = OtxToneSilenceDetectorStatus(g_hLogicDevs.mSilenceDetector.hHandle, &bSilence); 
      DemoCheckRc(g_hLogicDevs.mSilenceDetector.hHandle, result);
      
      // @parm OTX_TRUE if silence, OTX_FALSE if tone detected
      if (bSilence) {
        DemoLogf("s");
      } else {
        DemoLogf(".");
      }

    }
    
    //DemoLogf(".");
#endif
  }

END:
  
  stopTime = clock();

  DemoLogf(".\n");
  
  DemoLogf("DemoRawDataSender: Done (time=%2.1f seconds, Last nTaskRef=%d).\n", (double)(stopTime - startTime) / CLOCKS_PER_SEC, g_hLogicDevs.mRawDataSender.nTaskRef-1);
  
  return 0;

}



/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void  *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;


  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(g_hEventQueue, result);

      if (result == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (result == OTX_S_OK);
  }

  return 0;


}



/*******************************************************************************
  @func
    Prints a Driver Event. If HDLC Data, reads the data.
*******************************************************************************/
void DemoPrintEvent(
  OtxEventDataS *pEventData
)
{
  OTX_RESULT otxRc;
  char msgStr[256];

  DemoLogf("%.8X%.8X ", pEventData->m_nTimeStamp.sUint32.nHigh, pEventData->m_nTimeStamp.sUint32.nLow);

  switch (pEventData->m_eDeviceType) {
  case OTX_DEVICE_DSP:
    // Non-logical device related event from the DSPs
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      DemoLogf("DSP[%d] Event: %s (m_nParam=0x%X, m_nRequestId=0x%X)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nParam, pEventData->m_nRequestId);
    } else {
      DemoLogf("DSP[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_LDEVICE_TONE_FSK_DETECTOR:
    if (pEventData->m_nCode == OTX_FSK_DETECTOR_EC_BYTE) {
      DemoLogf("FskDetector: Octet = 0x%.2X\n", pEventData->m_nParam);
    } else if (pEventData->m_nCode == OTX_FSK_DETECTOR_EC_ALERT) {
      DemoLogf("FskDetector: ALERT Detected\n");
    } else if (pEventData->m_nCode == OTX_FSK_DETECTOR_EC_COMPLETE) {
      DemoLogf("FskDetector: Message completed\n");
    } else if (pEventData->m_nCode == OTX_FSK_DETECTOR_EC_TIMEOUT) {
      DemoLogf("FskDetector: Timeout. Incomplete message\n");
    } else if (pEventData->m_nCode == OTX_FSK_DETECTOR_EC_VERBOSE_STATUS) {
      DemoLogf("FskDetector: Verbose Status 0x%.8X\n", pEventData->m_nSourceId, pEventData->m_nParam);
    } else {
      DemoLogf("FskDetector: Unrecognized code (%X)\n", pEventData->m_nCode);
    }
    break;

  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }


}

/*******************************************************************************

  @func

  Processes User Commands

  @comm

*******************************************************************************/
OTX_INT32 DemoProcessUserCommand(char commandChar)
{

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?   Display Help text\n");
    DemoLogf(" h   Ditto\n");
    DemoLogf(" q   Quit\n");
    DemoLogf(" r   Record voice data from HS0\n");
    DemoLogf(" p   Playback voice data from file to HS0\n");
    break;

  case 'r':
    {
      char sqzFileName[64];
      char sqzReordingLength[64];

      DemoLogf("Voice Recorder. Please enter output file: ");
      OsaConsoleGetStr(sqzFileName);

      if ((g_hLogicDevs.mRawDataReceiver.pRxFile = fopen(sqzFileName, "wb")) == NULL) {
        DemoLogf("Error: Could not open file %s \n", sqzFileName);
        break;
      }

      DemoLogf("Please enter recording length (seconds): ");
      OsaConsoleGetStr(sqzReordingLength);
      g_hLogicDevs.mRawDataReceiver.nMsDuration = (clock_t) strtol(sqzReordingLength, NULL, 0) * 1000;

      DemoLogf("Creating Raw Data Receiver Thread (Recording for %d seconds)\n", g_hLogicDevs.mRawDataReceiver.nMsDuration/1000);
#if 0
      g_hLogicDevs.mRawDataReceiver.hThread = CreateThread(
        NULL,                 // Security attributes
        0,                    // Stack size
        DemoRawDataReceiver,  // pointer to thread function
        NULL,                 // argument for new thread
        0,                    // creation flags
        &g_hLogicDevs.mRawDataReceiver.nThreadId            // pointer to receive thread ID
        );
#else
      OsaThreadStart(DemoRawDataReceiver);
#endif
    }
    break;


  case 'p':
    {
      char sqzFileName[64];

      DemoLogf("Voice Playback. Please enter file to playback [%s]: ", DEMO_DEFAULT_PLAYBACK_FILE);
      OsaConsoleGetStr(sqzFileName);

      if (strlen(sqzFileName) == 0) {
        strcpy(sqzFileName, DEMO_DEFAULT_PLAYBACK_FILE);
      }

      if ((g_hLogicDevs.mRawDataSender.pTxFile = fopen(sqzFileName, "rb")) == NULL) {
        DemoLogf("Error: Could not open file %s \n", sqzFileName);
        break;
      }
    
      DemoLogf("Creating Raw Data Sender Thread\n");
#if 0
      g_hLogicDevs.mRawDataSender.hThread = CreateThread(
        NULL,                 // Security attributes
        0,                    // Stack size
        DemoRawDataSender,    // pointer to thread function
        NULL,                 // argument for new thread
        0,                    // creation flags
        &g_hLogicDevs.mRawDataSender.nThreadId            // pointer to receive thread ID
        );
#else
			OsaThreadStart(DemoRawDataSender);
#endif

    }
    break;


  case 'q':
  case 'Q':
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}


/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->logFileName       = DEMO_DEFAULT_LOG_FILE;
  envP->eLaw              = OTX_DATA_FORMAT_COMPAND8_A_LAW;
  envP->boardNo           = DEMO_DEFAULT_BOARD_NO;
  envP->eProgramModule    = OTX_DSP_SPM_OTXSPM1;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    OsaStdStrUpr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'L') {   // Log file
        envP->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'B') {   // Board number
        envP->boardNo = strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'U') {   // U-law Codec PCM Compression Mode
        envP->eLaw              = OTX_DATA_FORMAT_COMPAND8_U_LAW;
      } else if (*(argv[i]+1) == 'A') {   // A-law Codec PCM Compression Mode
        envP->eLaw              = OTX_DATA_FORMAT_COMPAND8_A_LAW;
      } else if (*(argv[i]+1) == 'P') {   // Dsp Program module
        if (*(argv[i]+2) == '1') {   // OtxSpm1.out
          envP->eProgramModule = OTX_DSP_SPM_OTXSPM1;
        } else if (*(argv[i]+2) == '2') {   // OtxSpm2.out
          envP->eProgramModule = OTX_DSP_SPM_OTXSPM2;
				}
      } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
				exit(1);
      }
    } else {
      DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
      DemoHelpPanel();
      OsaConsoleExit();
			exit(1);
    }
  }

}


/*******************************************************************************

  @func

  Displays a help for the command line.

  @comm

*******************************************************************************/
static void DemoHelpPanel()
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /B<boardNo>  : Board number (Default is %d)\n", DEMO_DEFAULT_BOARD_NO);
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /U           : U-law mode\n");
  DemoLogf(" /A           : A-law mode\n");
}


/*******************************************************************************

  @func

  Displays a message on the screen and prints it to the logfile.

  @comm

*******************************************************************************/
static void DemoLogf(char *formatStr, ...)
{
  va_list argList;
  va_start(argList, formatStr);

  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) vfprintf(g_pDemoLogFile,   formatStr, argList);

  va_end(argList);

}



/*******************************************************************************
*******************************************************************************/
int main(int argc, char *argv[])
{
  OTX_INT32 exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char commandChar;

	OsaConsoleInit();

  DemoLogf("T2P5X4FSK, %s, Odin TeleSystems Inc.\n", __DATE__);
  DemoLogf("Thor-2-PCI/Vidar-5x4-ASM Example program.\n");
  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);

  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    OsaConsoleExit();
		exit(-1);
  }

  DemoLogf("Log file is %s\n", g_demoEnv.logFileName);
  DemoLogf("Board number %d\n", g_demoEnv.boardNo);

  // Query Library revision
  {
    char szVersionInfo[255];
    OTX_BOOL bIsStaticLinkedLibrary;
    char szLibFileName[64];
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_REV_STR, szVersionInfo, 255); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueBool(OTX_LIB_HANDLE, OTX_ATTR_LIB_TYPE_STATIC, &bIsStaticLinkedLibrary); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_FILE_NAME, szLibFileName, 64); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    
    DemoLogf("Library version %s (filename %s, %s linked)\n", szVersionInfo, szLibFileName, bIsStaticLinkedLibrary ? "static":"dynamic");
  }


  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); 
  switch (OTX_RESULT_CODE(result)) {
  case OTX_RES_HANDLE:
    DemoLogf("Error: Could not connect to the OTX library.\nPossible cause: OTX Driver not loaded\n");
  	OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  case OTX_RES_NOT_SUPPORTED:
    DemoLogf("Error: Mismatch between OTX Driver revision and OTX Library revision.\nRevisions must match.\n");
  	OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  default:
    DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    break;
  }

  // Create application event queue
  {
    OTX_RESULT result;
    
    DemoLogf("Creating global event queue...\n");

    // Create a notification Event
    result = OtxDrvCreateEvent(&g_hNotificationEvent);
    if (result == OTX_E_NOTIMPL) {
      g_hNotificationEvent = NULL;    // D16V86 apps cannot use event objects
      result = OTX_S_OK;
    }
    
    DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    
    // Create Event Queue for driver notifications
    result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(g_hEventQueue, result);
    
    // Enable Event Queue
    result = OtxDrvEnable(g_hEventQueue, OTX_TASK_SYNC,0); DemoCheckRc(g_hEventQueue, result);
  }
  
  // Open Physical Devices
  {
    OTX_RESULT result;
    OTX_INT32 i;

    DemoLogf("Opening physical devices...\n");

    // Connect to the Physical Thor-2 Board Controller
    result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_THOR_PCI, g_demoEnv.boardNo, g_demoEnv.boardNo, g_hEventQueue, &g_hPhysDevs.hThorController0); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    
    // The T1/E1 devices are dummy devices in this demo
    for (i = 0; i < 2; i++) {
      DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d\n", i);
      result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_LI_T1E1, i, i, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    }

    // Get a handle to the Time-Space Switch (TSS) on the Thor-2-PCI board
    DemoLogf("Open OTX_DEVICE_TSS\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_TSS, 0, 0, g_hEventQueue, &g_hPhysDevs.hTss); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    
    // Get a handle to the Codec on the Thor-2-PCI board
    DemoLogf("Open OTX_DEVICE_CODEC\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_CODEC, 0, 0, g_hEventQueue, &g_hPhysDevs.hCodec); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    
    // Set PCM compression law for the Codec
    DemoLogf("Set Law to %s\n", g_demoEnv.eLaw == OTX_DATA_FORMAT_COMPAND8_A_LAW ? "A-law" : "U-law");
    result = OtxBrdCodecSetLaw(g_hPhysDevs.hCodec, g_demoEnv.eLaw); DemoCheckRc(g_hPhysDevs.hCodec, result);
    
    // Get a handle to to the ASM Daughter Board
    DemoLogf("Open OTX_DEVICE_VIDAR_5x_ASM\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController0, OTX_DEVICE_VIDAR_5x_ASM, 0, 0, g_hEventQueue, &g_hPhysDevs.hVidarAsmController0); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    
    // Get a handle to the first DSP on the ASM Daugther board
    DemoLogf("Open OTX_DEVICE_DSP #0\n");
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hVidarAsmController0, OTX_DEVICE_DSP, 0, 0, g_hEventQueue, &g_hPhysDevs.hDsp); DemoCheckRc(g_hPhysDevs.hVidarAsmController0, result);
    
    // Load DSP Program
    switch (g_demoEnv.eProgramModule) {
    case OTX_DSP_SPM_OTXSPM1:
      DemoLogf("Loading DSP program OTX_DSP_SPM_OTXSPM1 on DSP0\n");
      break;
    case OTX_DSP_SPM_OTXSPM2:
      DemoLogf("Loading DSP program OTX_DSP_SPM_OTXSPM2 on DSP0\n");
      break;
    default:
      DemoLogf("Unknown DSP program module (%d). Exiting.\n", g_demoEnv.eProgramModule);
      OtxDrvReleaseLib();
      OsaConsoleExit();
			exit(1);
    }
		result = OtxDspRunSpm(g_hPhysDevs.hDsp, g_demoEnv.eProgramModule); DemoCheckRc(g_hPhysDevs.hDsp, result);
    
  }

  // Setting clock source
  {
    DemoLogf("Setting clock source...\n");
    // Use Internal clock source
    result = OtxBrdSetClocks(g_hPhysDevs.hThorController0, OTX_CLOCK_MODE_LOCAL, OTX_CLOCK_SOURCE_INTERNAL); DemoCheckRc(g_hPhysDevs.hThorController0, result);
  }

  
  //    Enable Configured Physical Devices
  {
    OTX_RESULT result;

    DemoLogf("Enabling physical devices...\n");

    result = OtxDrvEnable(g_hPhysDevs.hThorController0,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hThorController0, result);
    result = OtxDrvEnable(g_hPhysDevs.hVidarAsmController0, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hVidarAsmController0, result);
    
    result = OtxDrvEnable(g_hPhysDevs.hDsp,            OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hDsp, result);
    result = OtxDrvEnable(g_hPhysDevs.hCodec,          OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hCodec, result);
  }
  
  // Create logical devices and its event queues
  {
    OTX_RESULT result;
    
    DemoLogf("Creating logical devices...\n");

    // Create a notification Event and Event Queue for the Raw Data Sender
    result = OtxDrvCreateEvent(&g_hLogicDevs.mRawDataSender.hNotificationEvent); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvCreateEventQueue(128, g_hLogicDevs.mRawDataSender.hNotificationEvent, &g_hLogicDevs.mRawDataSender.hEventQueue); DemoCheckRc(g_hLogicDevs.mRawDataSender.hEventQueue, result);
    result = OtxDrvEnable(g_hLogicDevs.mRawDataSender.hEventQueue, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender.hEventQueue, result);
    
    // Create Raw Data Sender
    result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp, OTX_LDEVICE_DATA_RAW_SENDER, g_hLogicDevs.mRawDataSender.hEventQueue, 0, &g_hLogicDevs.mRawDataSender.hHandle); DemoCheckRc(g_hPhysDevs.hDsp, result);
    
    // Create a notification Event and Event Queue for the Raw Data Receiver
    result = OtxDrvCreateEvent(&g_hLogicDevs.mRawDataReceiver.hNotificationEvent); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvCreateEventQueue(128, g_hLogicDevs.mRawDataReceiver.hNotificationEvent, &g_hLogicDevs.mRawDataReceiver.hEventQueue); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hEventQueue, result);
    result = OtxDrvEnable(g_hLogicDevs.mRawDataReceiver.hEventQueue, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hEventQueue, result);
    
    // Create Raw Data Receiver
    result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp, OTX_LDEVICE_DATA_RAW_RECEIVER, g_hLogicDevs.mRawDataReceiver.hEventQueue, 0, &g_hLogicDevs.mRawDataReceiver.hHandle); DemoCheckRc(g_hPhysDevs.hDsp, result);

    // Create Silence Detector
    result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp, OTX_LDEVICE_TONE_SILENCE_DETECTOR, g_hEventQueue, 0, &g_hLogicDevs.mSilenceDetector.hHandle); DemoCheckRc(g_hPhysDevs.hDsp, result);

    // Create FSK Detector
    result = OtxDrvCreateLogicalDevice(g_hPhysDevs.hDsp, OTX_LDEVICE_TONE_FSK_DETECTOR, g_hEventQueue, 0, &g_hLogicDevs.mFskDetector.hHandle); DemoCheckRc(g_hPhysDevs.hDsp, result);

    
  }
  
  // Connect pipes
  {
    OTX_RESULT result;
    OTX_UINT8 sqTimeSlotMask[32];

    DemoLogf("Connecting pipes...\n");

    
    // Pipes for the recording function (not the main function of this demo)
    result = OtxDrvCreatePipe(64, &g_hPipes.hCodecDsp); DemoCheckRc(g_hPipes.hCodecDsp, result);
    result = OtxDrvConnectPipe(g_hPipes.hCodecDsp, OTX_PIPE_INPUT,  g_hPhysDevs.hCodec, -1, NULL, 0); DemoCheckRc(g_hPipes.hCodecDsp, result);
    result = OtxDrvConnectPipe(g_hPipes.hCodecDsp, OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp,   -1, NULL, 0); DemoCheckRc(g_hPipes.hCodecDsp, result);
    // Create the following Connections in the receive direction:
    // Codec  ------> Raw Data Receiver
    result = OtxDrvConnectLogicalDevice(g_hPipes.hCodecDsp, -1, g_hLogicDevs.mRawDataReceiver.hHandle,       -1); DemoCheckRc(g_hPipes.hCodecDsp, result);
    

    // Pipe for the playback (Raw Data Sender)
    // First connect the pipe from the DSP to the LI on TS5
    result = OtxDrvCreatePipe(64, &g_hPipes.hDspCodec); DemoCheckRc(g_hPipes.hDspCodec, result);
    memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
    sqTimeSlotMask[5] = 0xFF;
    result = OtxDrvConnectPipe(g_hPipes.hDspCodec, OTX_PIPE_INPUT,  g_hPhysDevs.hDsp,  -1, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes.hDspCodec, result);
    result = OtxDrvConnectPipe(g_hPipes.hDspCodec, OTX_PIPE_OUTPUT, g_hPhysDevs.hLi[0],  -1, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes.hDspCodec, result);
    // Create the following Connections in the transmit direction:
    // Raw Data Sender [TS5] ------> Li [TS5] 
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.mRawDataSender.hHandle, -1, g_hPipes.hDspCodec, -1); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);

    { // Override the connections made by the driver
      OTX_HANDLE hPipe;
      result = OtxDrvCreatePipe(64, &hPipe); DemoCheckRc(hPipe, result);
      
      // DSP0 is on highway 10, the Raw Data sender was connected to TS5 in the section above
      memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
      sqTimeSlotMask[5] = 0xFF;
      result = OtxDrvConnectPipe(hPipe, OTX_PIPE_INPUT, g_hPhysDevs.hTss,  10, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes.hDspCodec, result);
      
      // Codec 0 is on highway 14, TS1
      memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
      sqTimeSlotMask[1] = 0xFF;
      result = OtxDrvConnectPipe(hPipe, OTX_PIPE_OUTPUT, g_hPhysDevs.hTss,  14, sqTimeSlotMask, 32); DemoCheckRc(hPipe, result);

      // Now we have the following connections in the transmit direction:
      // Raw Data Sender [TS5] ------> Codec0 

    }
    
    // Make the output from the Raw Data Sender also loop
    // back into the DSP (on a different timeslot) so that we
    // can use a silence detector and fsk detector to detect the
    // transmitted data
    {
      // First make a connection from the Li to the DSP so that we can 
      // tell the DSP which logical devices that needs to connect to this
      // timeslot [TS6]
      {
        OTX_HANDLE hPipe;

        result = OtxDrvCreatePipe(64, &hPipe); DemoCheckRc(hPipe, result);

        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[6] = 0xFF;
        result = OtxDrvConnectPipe(hPipe, OTX_PIPE_INPUT,  g_hPhysDevs.hLi[0],  -1, sqTimeSlotMask, 32); DemoCheckRc(hPipe, result);
        result = OtxDrvConnectPipe(hPipe, OTX_PIPE_OUTPUT, g_hPhysDevs.hDsp,  -1, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes.hDspCodec, result);
        
        // Connect the silence detector and the Fsk detector as the sink of this connection
        result = OtxDrvConnectLogicalDevice(hPipe, -1, g_hLogicDevs.mSilenceDetector.hHandle, -1); DemoCheckRc(g_hLogicDevs.mSilenceDetector.hHandle, result);
        result = OtxDrvConnectLogicalDevice(hPipe, -1, g_hLogicDevs.mFskDetector.hHandle, -1); DemoCheckRc(g_hLogicDevs.mFskDetector.hHandle, result);

        // Now we have the following connections in the receive direction:
        // LI [TS6] ---+---> DSP [TS6] Silence Detector
        //             |
        //             +---> DSP [TS6] Fsk Detector

      }

      { // Override the connections made by the driver to pick a different source
        // We switch out the Li source and replace it with the RawData Sender source
        // In effect we loop TS5 (from the DSP) back to TS6 (to the DSP)
        OTX_HANDLE hPipe;
        result = OtxDrvCreatePipe(64, &hPipe); DemoCheckRc(hPipe, result);
        
        // DSP0 is on highway 10
        // Raw Data Sender is on DSP 0 (highway 10), TS5
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[5] = 0xFF;
        result = OtxDrvConnectPipe(hPipe, OTX_PIPE_INPUT, g_hPhysDevs.hTss,  10, sqTimeSlotMask, 32); DemoCheckRc(hPipe, result);
        
        // Silence Detector is on DSP 0 (highway 10), TS6
        memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
        sqTimeSlotMask[6] = 0xFF;
        result = OtxDrvConnectPipe(hPipe, OTX_PIPE_OUTPUT, g_hPhysDevs.hTss,  10, sqTimeSlotMask, 32); DemoCheckRc(hPipe, result);

        // Now we have the following connections for the RawData Sender:
        //
        //                              +---> Codec0 
        //                              |                             
        // DSP [TS5] RawData Sender >---+---> DSP [TS6] Silence Detector
        //                              |
        //                              +---> DSP [TS6] Fsk Detector


      }
    
    }


    
    
  }
  
  // Init logical devices
  {
    OTX_RESULT result;
    
    DemoLogf("Initializing logical devices...\n");

    // Initialize the Raw Data Receivers (hosted in the DSP)
    result = OtxRawDataReceiverInit(
      g_hLogicDevs.mRawDataReceiver.hHandle,
      OTX_DATA_FORMAT_UINT8,  // No data conversion. OtxHdlc SPM only supports this format
      0xFF,        // Flag to be filtered
      OTX_FALSE,   // Disable Flag filtering
      OTX_TRUE     // Least Significant part is received first. Only applicable for a subchannel pipe (nCapacity < 64).
      ); DemoCheckRc(g_hLogicDevs.mRawDataReceiver.hHandle, result);


    // Initialize the Raw Data Senders (hosted in the DSP)
    result = OtxRawDataSenderInit(
      g_hLogicDevs.mRawDataSender.hHandle, 
      OTX_DATA_FORMAT_UINT8,  // No data conversion. Send octet by octet.
      0xFF,        // Flag to be sent when the sender is out of data
      OTX_TRUE     // Least Significant part is transmitted first. Only applicable for a subchannel pipe (nCapacity < 64).
      ); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);

    // Initialize Silence Detector
    result = OtxToneSilenceDetectorInit(g_hLogicDevs.mSilenceDetector.hHandle, g_demoEnv.eLaw, 2, 2000); DemoCheckRc(g_hLogicDevs.mSilenceDetector.hHandle, result);

    // Initialize FSK Detector
    result = OtxToneFskDetectorInit(g_hLogicDevs.mFskDetector.hHandle, g_demoEnv.eLaw); DemoCheckRc(g_hLogicDevs.mFskDetector.hHandle, result);

    
  }
  
  // Enable logical devices
  {
    OTX_RESULT result;

    DemoLogf("Enabling logical devices...\n");
    
    // Enable Raw Data Sender (always on)
    result = OtxDrvEnable(g_hLogicDevs.mRawDataSender.hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);

    // Enable Silence Detector (always on)
    result = OtxDrvEnable(g_hLogicDevs.mSilenceDetector.hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);

    // Enable FSK Detector (always on)
    result = OtxDrvEnable(g_hLogicDevs.mFskDetector.hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender.hHandle, result);


    // We enable the Raw Data Receiver only when it are in use (to avoid interrupting the host 
    
  }
  

  DemoLogf("Command loop running. Press '?' for help, 'q' to quit\n");

  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....
	OsaThreadStart(DemoIdleFunction);

  OsaThreadSleep(300);
  DemoLogf("\nPress 'h' for help\n");

  while (!exitFlag) {
    commandChar = getch();
    exitFlag = DemoProcessUserCommand(commandChar);
  }

  // Clean up
  OtxDrvReleaseLib();
	OsaConsoleExit();

  return 0;
}

